"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.deviceRepository = exports.DeviceRepository = void 0;
const concreteDevice_1 = require("./device/concreteDevice");
const deviceGroup_1 = require("./device/deviceGroup");
const deviceOverview_1 = require("./device/deviceOverview");
const instantiableBrowserDevice_1 = require("./device/instantiableBrowserDevice");
const instantiableCloudDevice_1 = require("./device/instantiableCloudDevice");
const service_common_1 = require("@crosslab/service-common");
const typeorm_1 = require("typeorm");
class DeviceRepository extends service_common_1.AbstractRepository {
    constructor() {
        super('Device');
        this.isInitialized = false;
    }
    initialize(AppDataSource) {
        deviceOverview_1.deviceOverviewRepository.initialize(AppDataSource);
        concreteDevice_1.concreteDeviceRepository.initialize(AppDataSource);
        deviceGroup_1.deviceGroupRepository.initialize(AppDataSource);
        instantiableBrowserDevice_1.instantiableBrowserDeviceRepository.initialize(AppDataSource);
        instantiableCloudDevice_1.instantiableCloudDeviceRepository.initialize(AppDataSource);
        this.isInitialized = true;
    }
    async create(data) {
        if (!this.isInitialized)
            this.throwUninitializedRepositoryError();
        switch (data.type) {
            case 'cloud instantiable':
                return await instantiableCloudDevice_1.instantiableCloudDeviceRepository.create(data);
            case 'device':
                return await concreteDevice_1.concreteDeviceRepository.create(data);
            case 'edge instantiable':
                return await instantiableBrowserDevice_1.instantiableBrowserDeviceRepository.create(data);
            case 'group':
                return await deviceGroup_1.deviceGroupRepository.create(data);
        }
    }
    async write(model, data) {
        switch (model.type) {
            case 'cloud instantiable':
                if (data.type !== model.type)
                    throw new service_common_1.InvalidValueError(`Model of type ${model.type} cannot be updated with data of type ${data.type}`, 400);
                return await instantiableCloudDevice_1.instantiableCloudDeviceRepository.write(model, data);
            case 'device':
                if (data.type !== model.type)
                    throw new service_common_1.InvalidValueError(`Model of type ${model.type} cannot be updated with data of type ${data.type}`, 400);
                return await concreteDevice_1.concreteDeviceRepository.write(model, data);
            case 'edge instantiable':
                if (data.type !== model.type)
                    throw new service_common_1.InvalidValueError(`Model of type ${model.type} cannot be updated with data of type ${data.type}`, 400);
                return await instantiableBrowserDevice_1.instantiableBrowserDeviceRepository.write(model, data);
            case 'group':
                if (data.type !== model.type)
                    throw new service_common_1.InvalidValueError(`Model of type ${model.type} cannot be updated with data of type ${data.type}`, 400);
                return await deviceGroup_1.deviceGroupRepository.write(model, data);
        }
    }
    async format(model, options) {
        switch (model.type) {
            case 'cloud instantiable':
                return await instantiableCloudDevice_1.instantiableCloudDeviceRepository.format(model);
            case 'device':
                return await concreteDevice_1.concreteDeviceRepository.format(model);
            case 'edge instantiable':
                return await instantiableBrowserDevice_1.instantiableBrowserDeviceRepository.format(model);
            case 'group':
                return await deviceGroup_1.deviceGroupRepository.format(model, options?.flatGroup);
        }
    }
    async save(model) {
        if (!this.isInitialized)
            this.throwUninitializedRepositoryError();
        switch (model.type) {
            case 'cloud instantiable':
                return await instantiableCloudDevice_1.instantiableCloudDeviceRepository.save(model);
            case 'device':
                return await concreteDevice_1.concreteDeviceRepository.save(model);
            case 'edge instantiable':
                return await instantiableBrowserDevice_1.instantiableBrowserDeviceRepository.save(model);
            case 'group':
                return await deviceGroup_1.deviceGroupRepository.save(model);
        }
    }
    async find(options) {
        if (!this.isInitialized)
            this.throwUninitializedRepositoryError();
        const foundDevices = [];
        const deviceOverviews = await deviceOverview_1.deviceOverviewRepository.find(options);
        const deviceTypes = [
            'device',
            'group',
            'edge instantiable',
            'cloud instantiable',
        ];
        const repositoryMapping = {
            'device': concreteDevice_1.concreteDeviceRepository,
            'group': deviceGroup_1.deviceGroupRepository,
            'edge instantiable': instantiableBrowserDevice_1.instantiableBrowserDeviceRepository,
            'cloud instantiable': instantiableCloudDevice_1.instantiableCloudDeviceRepository,
        };
        for (const type of deviceTypes) {
            const devices = deviceOverviews.filter((device) => device.type === type);
            if (devices.length > 0) {
                foundDevices.push(...(await repositoryMapping[type].find({
                    where: {
                        uuid: (0, typeorm_1.In)(devices.map((concreteDevice) => concreteDevice.uuid)),
                    },
                })));
            }
        }
        return foundDevices;
    }
    async findOne(options) {
        if (!this.isInitialized)
            this.throwUninitializedRepositoryError();
        const deviceOverview = await deviceOverview_1.deviceOverviewRepository.findOne(options);
        if (!deviceOverview)
            return null;
        switch (deviceOverview.type) {
            case 'cloud instantiable':
                return await instantiableCloudDevice_1.instantiableCloudDeviceRepository.findOne({
                    where: { uuid: deviceOverview.uuid },
                });
            case 'device':
                return await concreteDevice_1.concreteDeviceRepository.findOne({
                    where: { uuid: deviceOverview.uuid },
                });
            case 'edge instantiable':
                return await instantiableBrowserDevice_1.instantiableBrowserDeviceRepository.findOne({
                    where: { uuid: deviceOverview.uuid },
                });
            case 'group':
                return await deviceGroup_1.deviceGroupRepository.findOne({
                    where: { uuid: deviceOverview.uuid },
                });
        }
    }
    async findOneOrFail(options) {
        if (!this.isInitialized)
            this.throwUninitializedRepositoryError();
        const deviceOverview = await deviceOverview_1.deviceOverviewRepository.findOneOrFail(options);
        switch (deviceOverview.type) {
            case 'cloud instantiable':
                return await instantiableCloudDevice_1.instantiableCloudDeviceRepository.findOneOrFail({
                    where: { uuid: deviceOverview.uuid },
                });
            case 'device':
                return await concreteDevice_1.concreteDeviceRepository.findOneOrFail({
                    where: { uuid: deviceOverview.uuid },
                });
            case 'edge instantiable':
                return await instantiableBrowserDevice_1.instantiableBrowserDeviceRepository.findOneOrFail({
                    where: { uuid: deviceOverview.uuid },
                });
            case 'group':
                return await deviceGroup_1.deviceGroupRepository.findOneOrFail({
                    where: { uuid: deviceOverview.uuid },
                });
        }
    }
    async remove(model) {
        if (!this.isInitialized)
            this.throwUninitializedRepositoryError();
        switch (model.type) {
            case 'cloud instantiable':
                return await instantiableCloudDevice_1.instantiableCloudDeviceRepository.remove(model);
            case 'device':
                return await concreteDevice_1.concreteDeviceRepository.remove(model);
            case 'edge instantiable':
                return await instantiableBrowserDevice_1.instantiableBrowserDeviceRepository.remove(model);
            case 'group':
                return await deviceGroup_1.deviceGroupRepository.remove(model);
        }
    }
}
exports.DeviceRepository = DeviceRepository;
exports.deviceRepository = new DeviceRepository();
//# sourceMappingURL=device.js.map