"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.handleDeviceChangedEventCallback = void 0;
const peerconnection_1 = require("../../../database/repositories/peerconnection");
const types_1 = require("../../../generated/types");
const globals_1 = require("../../../globals");
const signaling_1 = require("../../../methods/signaling");
const service_common_1 = require("@crosslab/service-common");
/**
 * This function handles an incoming "device-changed" event callback.
 * @param callback The incoming "device-changed" callback to be handled.
 * @throws {MalformedBodyError} Thrown if the callback is malformed.
 * @returns The status code for the response to the incoming callback.
 */
async function handleDeviceChangedEventCallback(callback) {
    if (!callback.device) {
        throw new service_common_1.MalformedBodyError("Event-callbacks of type 'device-changed' require property 'device'", 400);
    }
    const device = callback.device;
    if (!(0, types_1.isConcreteDevice)(device, 'response') &&
        !(0, types_1.isDeviceGroup)(device, 'response') &&
        !(0, types_1.isInstantiableBrowserDevice)(device, 'response') &&
        !(0, types_1.isInstantiableCloudDevice)(device, 'response')) {
        throw new service_common_1.MalformedBodyError('Property "device" is not a valid device', 400);
    }
    if ((0, types_1.isConcreteDevice)(device, 'response')) {
        return await handleConcreteDevice(device);
    }
    else {
        return 410;
    }
}
exports.handleDeviceChangedEventCallback = handleDeviceChangedEventCallback;
async function handleConcreteDevice(concreteDevice) {
    const pendingConnectionsA = await peerconnection_1.peerconnectionRepository.find({
        where: {
            status: 'connecting',
            deviceA: {
                url: concreteDevice.url,
            },
        },
    });
    const pendingConnectionsB = await peerconnection_1.peerconnectionRepository.find({
        where: {
            status: 'connecting',
            deviceB: {
                url: concreteDevice.url,
            },
        },
    });
    const pendingConnections = [...pendingConnectionsA, ...pendingConnectionsB].filter((peerconnection, index, array) => array.findIndex((pc) => pc.uuid === peerconnection.uuid) === index);
    for (const pendingConnection of pendingConnections) {
        const deviceA = await globals_1.apiClient.getDevice(pendingConnection.deviceA.url);
        const deviceB = await globals_1.apiClient.getDevice(pendingConnection.deviceB.url);
        if (deviceA.connected && deviceB.connected) {
            clearTimeout(globals_1.timeoutMap.get(pendingConnection.uuid));
            signaling_1.signalingQueue.addPeerconnection(pendingConnection);
            globals_1.timeoutMap.delete(pendingConnection.uuid);
        }
    }
    return pendingConnections.length === 0 ? 410 : 200;
}
//# sourceMappingURL=deviceChanged.js.map