"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.websocketHandling = exports.connectedDevices = void 0;
const concreteDevice_1 = require("../../../database/repositories/device/concreteDevice");
const types_1 = require("../../../generated/types");
const callbacks_1 = require("../../../methods/callbacks");
const urlFromId_1 = require("../../../methods/urlFromId");
const messageHandling_1 = require("./messageHandling");
exports.connectedDevices = new Map();
class WebSocketConnectionError extends Error {
    constructor(message) {
        super(message);
        this.name = 'WebSocketConnectionError';
    }
}
/**
 * This function adds the /devices/ws endpoint, including its functionality, to an express application.
 * @param app The express application to add the /devices/ws endpoint to.
 */
function websocketHandling(app) {
    // TODO: close Peerconnections that have device as participant when websocket connection is closed?
    app.ws('/devices/websocket', (ws) => {
        // authenticate and start heartbeat
        ws.once('message', async (data) => {
            try {
                // device authentication and connection
                const message = JSON.parse(data.toString('utf8'));
                if (!((0, types_1.isMessage)(message) && (0, types_1.isAuthenticationMessage)(message))) {
                    return ws.close(1002, 'Received message is not an authentication message');
                }
                if (!message.token) {
                    return ws.close(1002, 'Authentication message does not contain a valid websocket token');
                }
                const deviceModel = await concreteDevice_1.concreteDeviceRepository.findOne({
                    where: { token: message.token },
                });
                if (!deviceModel) {
                    return ws.close(1002, 'No device found with matching websocket token');
                }
                deviceModel.connected = true;
                exports.connectedDevices.set(deviceModel.uuid, ws);
                await concreteDevice_1.concreteDeviceRepository.save(deviceModel);
                await (0, callbacks_1.sendChangedCallback)(deviceModel);
                console.log(`device '${(0, urlFromId_1.deviceUrlFromId)(deviceModel.uuid)}' connected`);
                // TODO: find out if this is really how it was intended
                ws.send(JSON.stringify({
                    messageType: 'authenticate',
                    authenticated: true,
                }));
                // heartbeat implementation
                let isAlive = true;
                ws.on('pong', () => {
                    isAlive = true;
                });
                const interval = setInterval(async function ping() {
                    try {
                        if (isAlive === false) {
                            deviceModel.connected = false;
                            await concreteDevice_1.concreteDeviceRepository.save(deviceModel);
                            await (0, callbacks_1.sendChangedCallback)(deviceModel);
                            exports.connectedDevices.delete(deviceModel.uuid);
                            clearInterval(interval);
                            return ws.terminate();
                        }
                        isAlive = false;
                        ws.ping();
                    }
                    catch (error) {
                        console.error(error);
                    }
                }, 30000);
                // close handler: stop heartbeat and disconnect device
                ws.on('close', async (code, reason) => {
                    clearInterval(interval);
                    exports.connectedDevices.delete(deviceModel.uuid);
                    if (code === 1002) {
                        console.error(new WebSocketConnectionError(reason.toString('utf-8')));
                    }
                });
                // message handler: handle incoming messages from devices
                ws.on('message', async (data) => {
                    try {
                        const message = JSON.parse(data.toString('utf-8'));
                        if (!(0, types_1.isMessage)(message)) {
                            ws.close(1002, 'Malformed Message');
                            return;
                        }
                        await (0, messageHandling_1.handleDeviceMessage)(deviceModel, message);
                    }
                    catch (error) {
                        console.error(error);
                    }
                });
            }
            catch (error) {
                console.error(error);
                return ws.close(1002, 'Something went wrong during authentication');
            }
        });
    });
}
exports.websocketHandling = websocketHandling;
//# sourceMappingURL=index.js.map