"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.postPeerconnections = void 0;
const peerconnection_1 = require("../../database/repositories/peerconnection");
const globals_1 = require("../../globals");
const callbacks_1 = require("../../methods/callbacks");
const signaling_1 = require("../../methods/signaling");
const service_common_1 = require("@crosslab/service-common");
/**
 * This function implements the functionality for handling POST requests on /peerconnections endpoint.
 * @param parameters The parameters of the request.
 * @param body The body of the request.
 * @param _user The user submitting the request.
 */
const postPeerconnections = async (parameters, body, _user) => {
    console.log(`postPeerconnections called`);
    const peerconnectionModel = await peerconnection_1.peerconnectionRepository.create(body);
    await peerconnection_1.peerconnectionRepository.save(peerconnectionModel);
    const deviceA = await globals_1.apiClient.getDevice(peerconnectionModel.deviceA.url);
    const deviceB = await globals_1.apiClient.getDevice(peerconnectionModel.deviceB.url);
    if (deviceA.type !== 'device' || deviceB.type !== 'device') {
        throw new service_common_1.InvalidValueError(`Cannot establish a peerconnection between devices of type '${deviceA.type}' and '${deviceB.type}'`, 400);
    }
    if (deviceA.connected && deviceB.connected) {
        // peerconnection can be started directly
        signaling_1.signalingQueue.addPeerconnection(peerconnectionModel);
    }
    else {
        // need to wait for devices to connect
        // register changed callbacks for devices to get notified when they connect
        const n_deviceA = await globals_1.apiClient.updateDevice(deviceA.url, { type: 'device' }, { changedUrl: callbacks_1.callbackUrl });
        const n_deviceB = await globals_1.apiClient.updateDevice(deviceB.url, { type: 'device' }, { changedUrl: callbacks_1.callbackUrl });
        // check that devices still have the correct type
        if (n_deviceA.type !== 'device' || n_deviceB.type !== 'device') {
            throw new service_common_1.InvalidValueError(`Cannot establish a peerconnection between devices of type '${deviceA.type}' and '${deviceB.type}'`, 400);
        }
        // set timeout for checking if devices are connected
        globals_1.timeoutMap.set(peerconnectionModel.uuid, setTimeout(async () => {
            try {
                console.log('devices did not connect');
                peerconnectionModel.status = 'failed';
                await peerconnection_1.peerconnectionRepository.save(peerconnectionModel);
                await (0, callbacks_1.sendStatusChangedCallback)(peerconnectionModel);
            }
            catch (error) {
                console.error(error);
            }
        }, 30000));
    }
    if (parameters.closedUrl) {
        console.log(`postPeerconnections: registering closed-callback for '${parameters.closedUrl}'`);
        const closedCallbackURLs = callbacks_1.closedCallbacks.get(peerconnectionModel.uuid) ?? [];
        closedCallbackURLs.push(parameters.closedUrl);
        callbacks_1.closedCallbacks.set(peerconnectionModel.uuid, closedCallbackURLs);
    }
    if (parameters.statusChangedUrl) {
        console.log(`postPeerconnections: registering status-changed-callback for '${parameters.statusChangedUrl}'`);
        const statusChangedCallbackURLs = callbacks_1.statusChangedCallbacks.get(peerconnectionModel.uuid) ?? [];
        statusChangedCallbackURLs.push(parameters.statusChangedUrl);
        callbacks_1.statusChangedCallbacks.set(peerconnectionModel.uuid, statusChangedCallbackURLs);
    }
    console.log(`postPeerconnections succeeded`);
    return {
        status: peerconnectionModel.status === 'connected' ? 201 : 202,
        body: await peerconnection_1.peerconnectionRepository.format(peerconnectionModel),
    };
};
exports.postPeerconnections = postPeerconnections;
//# sourceMappingURL=post.js.map